/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: CudaCannyEdgeDetection.h,v $
  Language:  C/C++ CUDA
  Date:      $Date: 2010-05-10 12:01:33 $
  Version:   $Revision: 2.2.1 $

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
/** \class CannyEdgeDetectionImageFilter
 * \brief CUDA implementation of Canny's algorithm
 * \author Luis Lourenco - Vision, Robotics, and Image Group (VRI)
 * http://www.inf.ufpr.br/vri/alumni/2011-LuisLourenco/
**/

// pixel types used on hysteresis
#define DEFINITIVE_EDGE 255
#define POSSIBLE_EDGE 128
#define NON_EDGE 0

// sizes for hysteresis' slice width
#define SLICE_WIDTH 18
#define SLICE_BLOCK_WIDTH 16

// there's a pixel modified or not on the slice
#define MODIFIED 1
#define NOT_MODIFIED 0

extern "C"
float* cuda2ndDerivativePos(dim3 DimGrid, dim3 DimBlock, const float *d_input, const float *d_Lvv, int width, int height);

extern "C"
float* cuda2ndDerivative(dim3 DimGrid, dim3 DimBlock, const float *d_input, int width, int height);

extern "C"
float* cudaHysteresis(dim3 DimGrid, dim3 DimBlock, float *d_img, float *d_gauss, int width, int height, float t1, float t2);
